/**
 * JASMINe
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JasmineEventSFBImpl.java 1153 2008-04-24 07:56:39Z loris $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eventswitch.beans.impl;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.ejb.Remote;
import javax.ejb.Stateless;

import org.osgi.framework.BundleContext;
import org.osgi.framework.InvalidSyntaxException;
import org.osgi.framework.ServiceEvent;
import org.osgi.framework.ServiceListener;
import org.osgi.framework.ServiceReference;
import org.ow2.easybeans.osgi.annotation.OSGiResource;
import org.ow2.jasmine.monitoring.eventswitch.beans.JasmineEventEB;
import org.ow2.jasmine.monitoring.eventswitch.beans.JasmineEventSFBRemote;
import org.ow2.jasmine.rules.api.IDroolsWorkingMemory;

/**
 * Bean to insert facts into the working memory.
 * @author Loris Bouzonnet
 */
@Stateless
@Remote(JasmineEventSFBRemote.class)
public class JasmineEventSFBImpl implements JasmineEventSFBRemote {

    /**
     * Bundle context that will be injected by the container.
     */
    @OSGiResource
    private BundleContext bundleContext = null;

    private ServiceListener workingMemoryListener;

    private IDroolsWorkingMemory droolsWorkingMemory;

    private ServiceReference  workingMemoryReference;

    /**
     * Bean startup call-back method.
     */
    @PostConstruct
    public void start() {
        workingMemoryListener = new ServiceListener() {

            public void serviceChanged(final ServiceEvent event) {
                ServiceReference ref = event.getServiceReference();
                // React on REGISTERED|UNREGISTERING events
                switch(event.getType()) {
                case ServiceEvent.REGISTERED:
                    droolsWorkingMemory = (IDroolsWorkingMemory) bundleContext.getService(ref);
                    break;
                case ServiceEvent.UNREGISTERING:
                    droolsWorkingMemory = null;
                    bundleContext.ungetService(ref);
                    break;
                }

            }
        };
        try {
            bundleContext.addServiceListener(
                    workingMemoryListener, "(objectClass=" + IDroolsWorkingMemory.class.getName() + ")");
        } catch (InvalidSyntaxException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        workingMemoryReference =
            bundleContext.getServiceReference(IDroolsWorkingMemory.class.getName());
        if(workingMemoryReference != null) {
            droolsWorkingMemory = (IDroolsWorkingMemory) bundleContext.getService(workingMemoryReference);
        }
    }

    /**
     * Bean stop call-back method.
     */
    @PreDestroy
    public void stop() {
        if(workingMemoryListener != null) {
            bundleContext.removeServiceListener(workingMemoryListener);
        }
        if(workingMemoryReference != null) {
            bundleContext.ungetService(workingMemoryReference);
        }
    }

    public void saveEvent(final JasmineEventEB e) {
        if(droolsWorkingMemory != null) {
            droolsWorkingMemory.insert(e);
        }
    }

}
